@@ -1,5 +1,5 @@
 /***************************************************************************************************
- * Copyright (c) 2023 - 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2023 - 2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -104,6 +104,7 @@ struct FragmentShuffler {
 ////////////////////////////////////////////////////////////////////////////////
 
 /// Partial specialization for `mma.sync` on 16b (F16/BF16) and `ldmatrix` on 8b (S8/U8)
+/// or for `mma.sync` on 8b (S8/U8) and `ldmatrix` on 4b (S4/U4)
 /// for operand A multiplicand going through upcasting. 
 template <
   /// Element type for the operand in registers for the mma.sync
@@ -122,8 +123,8 @@ struct FragmentShuffler <ElementMma_, ElementLoad_,
                          NumElementsInWarpFragment, 
                          NumElementsInMmaFragment,
                          Operand::kA,
-                         typename platform::enable_if<(sizeof_bits<ElementMma_>::value == 16) &&
-                                                 (sizeof_bits<ElementLoad_>::value == 8)>::type> {
+                         typename platform::enable_if<(sizeof_bits<ElementMma_>::value /
+                                                 sizeof_bits<ElementLoad_>::value == 2)>::type> {
 public:
   using ElementMma = ElementMma_;
   using ElementLoad = ElementLoad_;
@@ -187,6 +188,7 @@ public:
 ////////////////////////////////////////////////////////////////////////////////
 
 /// Partial specialization for `mma.sync` on 16b (F16/BF16) and `ldmatrix` on 8b (S8/U8)
+/// or for `mma.sync` on 8b (S8/U8) and `ldmatrix` on 4b (S4/U4)
 /// for operand B multiplicand going through upcasting. 
 template <
   /// Element type for the operand in registers for the mma.sync
@@ -205,8 +207,8 @@ struct FragmentShuffler <ElementMma_, ElementLoad_,
                          NumElementsInWarpFragment, 
                          NumElementsInMmaFragment,
                          Operand::kB,
-                         typename platform::enable_if<(sizeof_bits<ElementMma_>::value == 16) &&
-                                                 (sizeof_bits<ElementLoad_>::value == 8)>::type> {
+                         typename platform::enable_if<(sizeof_bits<ElementMma_>::value /
+                                                 sizeof_bits<ElementLoad_>::value == 2)>::type> {
 public:
   using ElementMma = ElementMma_;
   using ElementLoad = ElementLoad_;
@@ -374,10 +376,10 @@ public:
   using ArchMmaOperator = typename Policy::Operator;
 
   /// Underlying arch::Mma instruction datatype for A operand
-  using MmaElementA = typename ArchMmaOperator::ElementA;
+  using ElementAMma = typename ArchMmaOperator::ElementA;
 
   /// Underlying arch::Mma instruction datatype for B operand
-  using MmaElementB = typename ArchMmaOperator::ElementB;
+  using ElementBMma = typename ArchMmaOperator::ElementB;
 
   /// Underlying arch::Mma instruction datatype for C operand
   using MmaElementC = typename ArchMmaOperator::ElementC;
@@ -408,7 +410,7 @@ public:
 
   /// 
   // static int const kLoadShapeK = InstructionShape::kK * 
-  //  (sizeof_bits<MmaElementA>::value / sizeof_bits<ElementB>::value);
+  //  (sizeof_bits<ElementAMma>::value / sizeof_bits<ElementB>::value);
 
 public:
 
@@ -423,7 +425,7 @@ public:
 
   /// Storage for transformed A tile in registers (for use in Mma instruction)
   using TransformedFragmentA =
-      Array<MmaElementA, FragmentA::kElements>;
+      Array<ElementAMma, FragmentA::kElements>;
 
   /// Underlying arch::Mma instruction operand fragement for matrix A
   using MmaOperandA = typename ArchMmaOperator::FragmentA;
@@ -439,7 +441,7 @@ public:
 
   /// Storage for transformed B tile in registers (for use in Mma instruction)
   using TransformedFragmentB =
-      Array<MmaElementB, FragmentB::kElements>;
+      Array<ElementBMma, FragmentB::kElements>;
 
   /// Underlying arch::Mma instruction operand fragement for matrix B
   using MmaOperandB = typename ArchMmaOperator::FragmentB;
@@ -523,13 +525,13 @@ public:
                  FragmentA const &A, FragmentB const &B) const {
 
     // Shuffle data within warp to obtain the mma.sync operand layout
-    detail::FragmentShuffler<MmaElementB, ElementB, MmaIterations::kColumn, 
+    detail::FragmentShuffler<ElementBMma, ElementB, MmaIterations::kColumn, 
              FragmentB::kElements, MmaOperandB::kElements, Operand::kB> shuffler_B;
     FragmentB tmp_B; 
     tmp_B = shuffler_B(B);
 
     // Convert the B operand to the Mma Instruction operand type
-    detail::FragmentConverter<MmaElementB, ElementB, FragmentB::kElements> convert_B;
+    detail::FragmentConverter<ElementBMma, ElementB, FragmentB::kElements> convert_B;
     dst_B = convert_B(tmp_B);
 
     FragmentA tmp_A;
@@ -537,16 +539,16 @@ public:
     Array<ElementA, FragmentA::kElements / 2> *
         ptr_tmp_A = reinterpret_cast<Array<ElementA,
                                              FragmentA::kElements / 2> *>(&tmp_A);
-    Array<MmaElementA, FragmentA::kElements / 2> *
-        ptr_dst_A = reinterpret_cast<Array<MmaElementA,
+    Array<ElementAMma, FragmentA::kElements / 2> *
+        ptr_dst_A = reinterpret_cast<Array<ElementAMma,
                                              FragmentA::kElements / 2> *>(&dst_A);
 
     // Shuffle data within warp to obtain the mma.sync operand layout
-    detail::FragmentShuffler<MmaElementA, ElementA, MmaIterations::kRow,
+    detail::FragmentShuffler<ElementAMma, ElementA, MmaIterations::kRow,
              FragmentA::kElements, MmaOperandA::kElements, Operand::kA> shuffler_A;
 
     // Convert the A operand to the Mma Instruction operand type
-    detail::FragmentConverter<MmaElementA, ElementA, FragmentA::kElements / 2> convert_A;
+    detail::FragmentConverter<ElementAMma, ElementA, FragmentA::kElements / 2> convert_A;
 
     tmp_A = shuffler_A(A);
     ptr_dst_A[0] = convert_A(ptr_tmp_A[0]);
